@@ -23,11 +23,9 @@
 #include <config.h>
 #endif
 
-#include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <arpa/inet.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <netinet/in.h>
@@ -40,17 +38,11 @@
 
 #include <pulsecore/core-error.h>
 #include <pulsecore/core-util.h>
-#include <pulsecore/socket-util.h>
 #include <pulsecore/log.h>
 #include <pulsecore/macro.h>
 #include <pulsecore/strbuf.h>
 #include <pulsecore/ioline.h>
-
-#ifdef HAVE_POLL_H
-#include <poll.h>
-#else
-#include <pulsecore/poll.h>
-#endif
+#include <pulsecore/arpa-inet.h>
 
 #include "rtsp_client.h"
 
@@ -60,7 +52,6 @@ struct pa_rtsp_client {
     uint16_t port;
 
     pa_socket_client *sc;
-    pa_iochannel *io;
     pa_ioline *ioline;
 
     pa_rtsp_cb_t callback;
@@ -111,10 +102,8 @@ void pa_rtsp_client_free(pa_rtsp_client* c) {
 
     if (c->sc)
         pa_socket_client_unref(c->sc);
-    if (c->ioline)
-        pa_ioline_close(c->ioline);
-    else if (c->io)
-        pa_iochannel_free(c->io);
+
+    pa_rtsp_disconnect(c);
 
     pa_xfree(c->hostname);
     pa_xfree(c->url);
@@ -187,7 +176,6 @@ static void line_callback(pa_ioline *line, const char *s, void *userdata) {
     if (!s) {
         /* Keep the ioline/iochannel open as they will be freed automatically */
         c->ioline = NULL;
-        c->io = NULL;
         c->callback(c, STATE_DISCONNECTED, NULL, c->userdata);
         return;
     }
@@ -303,8 +291,7 @@ static void on_connection(pa_socket_client *sc, pa_iochannel *io, void *userdata
         pa_log("Connection failed: %s", pa_cstrerror(errno));
         return;
     }
-    pa_assert(!c->io);
-    c->io = io;
+    pa_assert(!c->ioline);
 
     c->ioline = pa_ioline_new(io);
     pa_ioline_set_callback(c->ioline, line_callback, c);
@@ -337,6 +324,7 @@ int pa_rtsp_connect(pa_rtsp_client *c) {
     pa_xfree(c->session);
     c->session = NULL;
 
+    pa_log_debug("Attempting to connect to server '%s:%d'", c->hostname, c->port);
     if (!(c->sc = pa_socket_client_new_string(c->mainloop, TRUE, c->hostname, c->port))) {
         pa_log("failed to connect to server '%s:%d'", c->hostname, c->port);
         return -1;
@@ -360,9 +348,6 @@ void pa_rtsp_disconnect(pa_rtsp_client *c) {
 
     if (c->ioline)
         pa_ioline_close(c->ioline);
-    else if (c->io)
-        pa_iochannel_free(c->io);
-    c->io = NULL;
     c->ioline = NULL;
 }
 
@@ -385,8 +370,7 @@ void pa_rtsp_set_url(pa_rtsp_client* c, const char* url) {
     c->url = pa_xstrdup(url);
 }
 
-void pa_rtsp_add_header(pa_rtsp_client *c, const char* key, const char* value)
-{
+void pa_rtsp_add_header(pa_rtsp_client *c, const char* key, const char* value) {
     pa_assert(c);
     pa_assert(key);
     pa_assert(value);
@@ -394,8 +378,7 @@ void pa_rtsp_add_header(pa_rtsp_client *c, const char* key, const char* value)
     pa_headerlist_puts(c->headers, key, value);
 }
 
-void pa_rtsp_remove_header(pa_rtsp_client *c, const char* key)
-{
+void pa_rtsp_remove_header(pa_rtsp_client *c, const char* key) {
     pa_assert(c);
     pa_assert(key);
 
@@ -408,13 +391,11 @@ static int rtsp_exec(pa_rtsp_client* c, const char* cmd,
                         pa_headerlist* headers) {
     pa_strbuf* buf;
     char* hdrs;
-    ssize_t l;
 
     pa_assert(c);
     pa_assert(c->url);
-
-    if (!cmd)
-        return -1;
+    pa_assert(cmd);
+    pa_assert(c->ioline);
 
     pa_log_debug("Sending command: %s", cmd);
 
@@ -453,11 +434,9 @@ static int rtsp_exec(pa_rtsp_client* c, const char* cmd,
     hdrs = pa_strbuf_tostring_free(buf);
     /*pa_log_debug("Submitting request:");
     pa_log_debug(hdrs);*/
-    l = pa_iochannel_write(c->io, hdrs, strlen(hdrs));
+    pa_ioline_puts(c->ioline, hdrs);
     pa_xfree(hdrs);
 
-    /* FIXME: this is broken, not necessarily all bytes are written */
-
     return 0;
 }
 
